Class: MotionKit::BaseLayout

Inherits:
Object
  • Object
show all
Extended by:
BaseLayoutClassMethods
Defined in:
lib/motion-kit/layouts/base_layout.rb,
lib/motion-kit-ios/layouts/layout_device.rb,
lib/motion-kit-cocoa/layouts/sugarcube_compat.rb,
lib/motion-kit-cocoa/layouts/accessibility_compat.rb

Overview

Abstract base class, responsible for "registration" of layout classes with the class targets method.

Very few methods are defined on BaseLayout, and any unknown methods are delegated to the 'apply' method, which accepts a method name, arguments, and an optional block to set the new context.

The TreeLayout subclass defines methods that are appropriate for adding and removing views to a view hierarchy.

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from BaseLayoutClassMethods

layout_for, memoize, new_child, target_klasses, targets

Constructor Details

#initialize(args = {}) ⇒ BaseLayout

Returns a new instance of BaseLayout.



19
20
21
22
23
24
25
26
27
28
29
30
31
# File 'lib/motion-kit/layouts/base_layout.rb', line 19

def initialize(args={})
  # @layout is the object we look in for style methods
  @layout = self
  # the Layout object that implements custom style methods. Leave this as nil
  # in the initializer.
  @layout_delegate = nil
  @layout_state = :initial
  # You can set a root view by using .new(root: some_view)
  # Explicit roots will not have a strong reference from
  # MotionKit, so retain one yourself from your controller
  # or other view to prevent deallocation.
  @preset_root = WeakRef.new(args[:root])
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(method_name, *args, &block) ⇒ Object

Methods that style the view start out as missing methods. This just calls 'apply', which searches for the method in the delegate (@layout_delegate) or using inspection (respond_to?(:setFoo)).

Examples:

def 
  frame [[0, 0], [100, 20]]
  title 'Login'
end


156
157
158
# File 'lib/motion-kit/layouts/base_layout.rb', line 156

def method_missing(method_name, *args, &block)
  self.apply(method_name, *args, &block)
end

Instance Attribute Details

#parent ⇒ Object (readonly)

Returns the value of attribute parent.



17
18
19
# File 'lib/motion-kit/layouts/base_layout.rb', line 17

def parent
  @parent
end

Class Method Details

.delegate_method_fix(method_name) ⇒ Object

Prevents infinite loops when methods that are defined on Object/Kernel are not properly delegated to the target.



278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
# File 'lib/motion-kit/layouts/base_layout.rb', line 278

def delegate_method_fix(method_name)
  running_name = "motion_kit_is_calling_#{method_name}"
  define_method(method_name) do |*args, &block|
    if target.motion_kit_meta[running_name]
      if block
        apply_with_context(method_name, *args, &block)
      else
        apply_with_target(method_name, *args)
      end
    else
      target.motion_kit_meta[running_name] = true
      retval = apply(method_name, *args)
      target.motion_kit_meta[running_name] = false
      return retval
    end
  end
end

Instance Method Details

#add_deferred_block(context, &block) ⇒ Object

Only intended for private use



115
116
117
118
119
120
121
122
123
124
125
126
# File 'lib/motion-kit/layouts/base_layout.rb', line 115

def add_deferred_block(context, &block)
  context ||= @context
  if context.nil? && @assign_root
    context ||= self.create_default_root_context
  end
  raise InvalidDeferredError.new('deferred must be run inside of a context') unless context
  raise ArgumentError.new('Block required') unless block

  self.deferred_blocks << [context, block]

  self
end

#apply(method_name, *args, &block) ⇒ Object

Tries to call the setter (foo 'value' => view.setFoo('value')), or assignment method (foo 'value' => view.foo = 'value'), or if a block is given, then the object returned by 'method_name' is assigned as the new context, and the block is executed.

You can call this method directly, but usually it is called via method_missing.

Raises:



189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
# File 'lib/motion-kit/layouts/base_layout.rb', line 189

def apply(method_name, *args, &block)
  method_name = method_name.to_s
  raise ApplyError.new("Cannot apply #{method_name.inspect} to instance of #{target.class.name}") if method_name.length == 0

  # if there is no target, than we should raise the NoMethodError; someone
  # called a method on the layout directly.
  begin
    target = self.target
  rescue NoContextError => e
    raise NoMethodError.new("undefined method `#{method_name}' for #{self}:#{self.class}", method_name)
  end

  objc_method_name, objc_method_args = objc_version(method_name, args)
  ruby_method_name = ruby_version(method_name)

  @layout_delegate ||= Layout.layout_for(@layout, target.class)
  if objc_method_name && @layout_delegate.respond_to?(objc_method_name)
    return @layout_delegate.send(objc_method_name, *objc_method_args, &block)
  elsif @layout_delegate.respond_to?(ruby_method_name)
    return @layout_delegate.send(ruby_method_name, *args, &block)
  end

  if block
    apply_with_context(method_name, *args, &block)
  else
    apply_with_target(method_name, *args, &block)
  end
end

#apply_with_context(method_name, *args, &block) ⇒ Object



218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
# File 'lib/motion-kit/layouts/base_layout.rb', line 218

def apply_with_context(method_name, *args, &block)
  objc_method_name, objc_method_args = objc_version(method_name, args)
  ruby_method_name = ruby_version(method_name)

  if objc_method_name && target.respond_to?(objc_method_name)
    new_context = target.send(objc_method_name, *objc_method_args)
    self.context(new_context, &block)
  elsif target.respond_to?(ruby_method_name)
    new_context = target.send(ruby_method_name, *args)
    self.context(new_context, &block)
  elsif ruby_method_name.include?('_')
    objc_name = MotionKit.objective_c_method_name(ruby_method_name)
    self.apply(objc_name, *args, &block)
  else
    raise ApplyError.new("Cannot apply #{ruby_method_name.inspect} to instance of #{target.class.name}")
  end
end

#apply_with_target(method_name, *args, &block) ⇒ Object



236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
# File 'lib/motion-kit/layouts/base_layout.rb', line 236

def apply_with_target(method_name, *args, &block)
  objc_method_name, objc_method_args = objc_version(method_name, args)
  ruby_method_name = ruby_version(method_name)

  setter = MotionKit.setter(ruby_method_name)
  assign = "#{ruby_method_name}="

  # The order is important here.
  # - unchanged method name if no args are passed (e.g. `layer`)
  # - setter (`setLayer(val)`)
  # - assign (`layer=val`)
  # - unchanged method name *again*, because many Ruby classes provide a
  #   combined getter/setter (`layer(val)`)
  # - lastly, try again after converting to camelCase
  if objc_method_name && target.respond_to?(objc_method_name)
    target.send(objc_method_name, *objc_method_args, &block)
  elsif args.empty? && target.respond_to?(ruby_method_name)
    target.send(ruby_method_name, *args, &block)
  elsif target.respond_to?(setter)
    target.send(setter, *args, &block)
  elsif target.respond_to?(assign)
    target.send(assign, *args, &block)
  elsif target.respond_to?(ruby_method_name)
    target.send(ruby_method_name, *args, &block)
  # UIAppearance classes are a whole OTHER thing; they never return 'true'
  elsif target.is_a?(MotionKit.appearance_class)
    target.send(setter, *args, &block)
  # Finally, try again with camel case if there's an underscore.
  elsif method_name.include?('_')
    objc_name = MotionKit.objective_c_method_name(method_name)
    self.apply(objc_name, *args, &block)
  else
    target.send(setter, *args, &block)
  end
end

#context(target, &block) ⇒ Object

Runs a block of code with a new object as the 'context'. Methods from the Layout classes are applied to this target object, and missing methods are delegated to a new Layout instance that is created based on the new context.

This method is part of the public API, you can pass in any object to have it become the 'context'.

Example: def table_view_style content = target.contentView if content context(content) do background_color UIColor.clearColor end end

  # usually you use 'context' automatically via method_missing, by
  # passing a block to a method that returns an object. That object becomes
  # the new context.
  layer do
    # self is now a CALayerLayout instance
    corner_radius 5
  end
end


75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
# File 'lib/motion-kit/layouts/base_layout.rb', line 75

def context(target, &block)
  return target unless block
  # this little line is incredibly important; the context is only set on
  # the top-level Layout object.
  return @layout.context(target, &block) if @layout != self

  if target.is_a?(Symbol)
    target = self.get(target)
  end

  context_was, parent_was, delegate_was = @context, @parent, @layout_delegate

  prev_should_run = @should_run_deferred
  if @should_run_deferred.nil?
    @should_run_deferred = true
  else
    @should_run_deferred = false
  end
  @parent = MK::Parent.new(context_was)
  @context = target
  @context.motion_kit_meta[:delegate] ||= Layout.layout_for(@layout, @context.class)
  @layout_delegate = @context.motion_kit_meta[:delegate]
  yield
  @layout_delegate, @context, @parent = delegate_was, context_was, parent_was
  if @should_run_deferred
    run_deferred(target)
  end
  @should_run_deferred = prev_should_run

  target
end

#deferred(context = nil, &block) ⇒ Object

Blocks passed to deferred are run at the end of a "session", usually after a call to Layout#layout.



109
110
111
112
# File 'lib/motion-kit/layouts/base_layout.rb', line 109

def deferred(context=nil, &block)
  context ||= @context
  return @layout.add_deferred_block(context, &block)
end

#deferred_blocks ⇒ Object

Only intended for private use



129
130
131
# File 'lib/motion-kit/layouts/base_layout.rb', line 129

def deferred_blocks
  @deferred_blocks ||= []
end

#ipad? ⇒ Boolean

Returns:

  • (Boolean)


17
18
19
# File 'lib/motion-kit-ios/layouts/layout_device.rb', line 17

def ipad?
  UIDevice.currentDevice.userInterfaceIdiom == UIUserInterfaceIdiomPad
end

#iphone35? ⇒ Boolean

Returns:

  • (Boolean)


13
14
15
# File 'lib/motion-kit-ios/layouts/layout_device.rb', line 13

def iphone35?
  iphone? && UIScreen.mainScreen.bounds.size.width == 320 && UIScreen.mainScreen.bounds.size.height == 480
end

#iphone4? ⇒ Boolean

Returns:

  • (Boolean)


9
10
11
# File 'lib/motion-kit-ios/layouts/layout_device.rb', line 9

def iphone4?
  iphone? && UIScreen.mainScreen.bounds.size.width == 320 && UIScreen.mainScreen.bounds.size.height == 568
end

#iphone? ⇒ Boolean

Returns:

  • (Boolean)


5
6
7
# File 'lib/motion-kit-ios/layouts/layout_device.rb', line 5

def iphone?
  UIDevice.currentDevice.userInterfaceIdiom == UIUserInterfaceIdiomPhone
end

#objc_version(method_name, args) ⇒ Object



160
161
162
163
164
165
166
167
168
169
170
171
172
# File 'lib/motion-kit/layouts/base_layout.rb', line 160

def objc_version(method_name, args)
  if method_name.count(':') > 1
    objc_method_name = method_name
    objc_method_args = [args[0]].concat args[1].values
  elsif args.length == 2 && args[1].is_a?(Hash) && !args[1].empty?
    objc_method_name = "#{method_name}:#{args[1].keys.join(':')}:"
    objc_method_args = [args[0]].concat args[1].values
  else
    objc_method_name = nil
    objc_method_args = nil
  end
  return objc_method_name, objc_method_args
end

#retina? ⇒ Boolean

Returns:

  • (Boolean)


21
22
23
# File 'lib/motion-kit-ios/layouts/layout_device.rb', line 21

def retina?
  UIScreen.mainScreen.respond_to?(:scale) && UIScreen.mainScreen.scale == 2
end

#ruby_version(method_name) ⇒ Object



174
175
176
177
178
179
180
# File 'lib/motion-kit/layouts/base_layout.rb', line 174

def ruby_version(method_name)
  if method_name.count(':') > 1
    method_name.split(':').first
  else
    method_name
  end
end

#run_deferred(top_level_context) ⇒ Object

Only intended for private use



134
135
136
137
138
139
140
141
142
143
144
145
# File 'lib/motion-kit/layouts/base_layout.rb', line 134

def run_deferred(top_level_context)
  deferred_blocks = self.deferred_blocks
  @deferred_blocks = nil

  deferred_blocks.each do |target, block|
    context(target, &block)
  end

  if @deferred_blocks
    run_deferred(top_level_context)
  end
end

#set_layout(layout) ⇒ Object



33
34
35
# File 'lib/motion-kit/layouts/base_layout.rb', line 33

def set_layout(layout)
  @layout = WeakRef.new(layout)
end

#target ⇒ Object



37
38
39
40
41
42
43
44
45
46
47
# File 'lib/motion-kit/layouts/base_layout.rb', line 37

def target
  if @layout.nil? || @layout == self
    # only the "root layout" instance is allowed to change the context.
    # if there isn't a context set, try and create a root instance; this
    # will fail if we're not in a state that allows the root to be created
    @context ||= create_default_root_context
  else
    # child layouts get the context from the root layout
    @layout.target
  end
end

#v ⇒ Object



48
# File 'lib/motion-kit/layouts/base_layout.rb', line 48

def v ; target ; end